from starlette import status
from starlette.exceptions import HTTPException

from rowantree.contracts import UserFeature, UserIncome, UserNotification, UserStore
from rowantree.contracts.dto.user.state import UserState

from ..db.incorrect_row_count_error import IncorrectRowCountError
from .abstract_controller import AbstractController


class UserStateGetController(AbstractController):
    def execute(self, user_guid: str) -> UserState:
        # User Game State
        try:
            active: bool = self.dao.user_active_state_get(user_guid=user_guid)
        except IncorrectRowCountError as error:
            raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="User not found") from error

        # User Stores (Inventory)
        stores: list[UserStore] = self.dao.user_stores_get(user_guid=user_guid)

        # User Income
        income: list[UserIncome] = self.dao.user_income_get(user_guid=user_guid)

        # Features
        features: list[str] = self.dao.user_features_get(user_guid=user_guid)

        # Population
        population: int = self.dao.user_population_by_guid_get(user_guid=user_guid)

        # Active Feature
        active_features: UserFeature = self.dao.user_active_feature_get(user_guid=user_guid)

        # Active Feature Details
        active_features_details: UserFeature = self.dao.user_active_feature_state_details_get(user_guid=user_guid)

        # Merchants
        merchants: list[str] = self.dao.user_merchant_transforms_get(user_guid=user_guid)

        # Notifications
        notifications: list[UserNotification] = self.dao.user_notifications_get(user_guid=user_guid)

        user_state: UserState = UserState(
            active=active,
            stores=stores,
            income=income,
            features=features,
            active_features=active_features,
            active_features_details=active_features_details,
            population=population,
            merchants=merchants,
            notifications=notifications,
        )
        return user_state



import logging
import socket

from mysql.connector import Error, errorcode
from mysql.connector.pooling import MySQLConnectionPool

from ..config.server import ServerConfig


# pylint: disable=duplicate-code
def get_connect_pool(config: ServerConfig) -> MySQLConnectionPool:
    try:
        logging.debug("Connecting to database")
        cnxpool: MySQLConnectionPool = MySQLConnectionPool(
            pool_name="servercnxpool",
            pool_size=32,
            user=config.database_username,
            password=config.database_password,
            host=config.database_server,
            database=config.database_name,
        )
        return cnxpool
    except socket.error as error:
        logging.debug(error)
        raise error
    except Error as error:
        if error.errno == errorcode.ER_ACCESS_DENIED_ERROR:
            logging.debug("Something is wrong with your user name or password")
        elif error.errno == errorcode.ER_BAD_DB_ERROR:
            logging.debug("Database does not exist")
        else:
            logging.debug(error)
        raise error



from ..contracts.responses.user_merchant_transforms_get_response import UserMerchantTransformsGetResponse
from .abstract_controller import AbstractController


class UserMerchantTransformsGetController(AbstractController):
    def execute(self, user_guid: str) -> UserMerchantTransformsGetResponse:
        return UserMerchantTransformsGetResponse(merchants=self.dao.user_merchant_transforms_get(user_guid=user_guid))



from typing import Any

from .abstract_controller import AbstractController


class UserDeleteController(AbstractController):
    def execute(self, user_guid: str) -> Any:
        return self.dao.user_delete(user_guid=user_guid)



from rowantree.contracts import WorldStatus

from ..controllers.abstract_controller import AbstractController


class WorldStatusGetController(AbstractController):
    def execute(self) -> WorldStatus:
        return WorldStatus(active_users=self.dao.users_active_get())



from ..contracts.requests.user_income_set_request import UserIncomeSetRequest
from ..controllers.abstract_controller import AbstractController


class UserIncomeSetController(AbstractController):
    def execute(self, user_guid: str, request: UserIncomeSetRequest) -> None:
        self.dao.user_income_set(user_guid=user_guid, transaction=request)



from ..contracts.requests.merchant_transform_request import MerchantTransformRequest
from .abstract_controller import AbstractController


class MerchantTransformPerformController(AbstractController):
    def execute(self, user_guid: str, request: MerchantTransformRequest) -> None:
        self.dao.merchant_transform_perform(user_guid=user_guid, store_name=request.store_name)



from ..contracts.responses.user_features_get_response import UserFeaturesGetResponse
from .abstract_controller import AbstractController


class UserFeaturesGetController(AbstractController):
    def execute(self, user_guid: str) -> UserFeaturesGetResponse:
        return UserFeaturesGetResponse(features=self.dao.user_features_get(user_guid=user_guid))



from ..contracts.responses.user_income_get_response import UserIncomeGetResponse
from .abstract_controller import AbstractController


class UserIncomeGetController(AbstractController):
    def execute(self, user_guid: str) -> UserIncomeGetResponse:
        return UserIncomeGetResponse(income=self.dao.user_income_get(user_guid=user_guid))



from rowantree.contracts import UserFeature

from .abstract_controller import AbstractController


class UserFeaturesActiveGetController(AbstractController):
    def execute(self, user_guid: str, details: bool) -> UserFeature:
        if details:
            feature: UserFeature = self.dao.user_active_feature_state_details_get(user_guid=user_guid)
        else:
            feature: UserFeature = self.dao.user_active_feature_get(user_guid=user_guid)
        return feature



from rowantree.contracts import ActionQueue

from .abstract_controller import AbstractController


class ActionQueueProcessController(AbstractController):
    def execute(self, action_queue: ActionQueue) -> None:
        self.dao.process_action_queue(action_queue=action_queue)



""" Database DAO Definition """

import logging
import socket
from datetime import datetime
from typing import Optional, Tuple

import mysql.connector
from mysql.connector import errorcode
from mysql.connector.pooling import MySQLConnectionPool

from rowantree.contracts import ActionQueue, UserEvent, UserFeature, UserIncome, UserNotification, UserStore

from ..contracts.requests.user_income_set_request import UserIncomeSetRequest
from .incorrect_row_count_error import IncorrectRowCountError


class DBDAO:
    """
    Database DAO

    Attributes
    ----------
    cnxpool: Any
        MySQL Connection Pool
    """

    cnxpool: MySQLConnectionPool

    def __init__(self, cnxpool: MySQLConnectionPool):
        self.cnxpool = cnxpool

    def merchant_transform_perform(self, user_guid: str, store_name: str) -> None:
        args: list = [user_guid, store_name]
        self._call_proc("peformMerchantTransformByGUID", args)

    def user_active_feature_get(self, user_guid: str) -> UserFeature:
        args: list = [
            user_guid,
        ]
        rows: list[Tuple[str]] = self._call_proc("getUserActiveFeatureByGUID", args)
        if len(rows) != 1:
            raise IncorrectRowCountError(f"Result count was not exactly one. Received: {rows}")
        feature_detail: UserFeature = UserFeature(name=rows[0][0])
        return feature_detail

    def user_active_feature_state_details_get(self, user_guid: str) -> UserFeature:
        args: list = [
            user_guid,
        ]
        rows: list[Tuple[str, Optional[str]]] = self._call_proc("getUserActiveFeatureStateDetailsByGUID", args)
        if len(rows) != 1:
            raise IncorrectRowCountError(f"Result count was not exactly one. Received: {rows}")

        feature_detail: UserFeature = UserFeature(name=rows[0][0], description=rows[0][1])
        return feature_detail

    def users_active_get(self) -> list[str]:
        my_active_users: list[str] = []
        rows: list[Tuple] = self._call_proc("getActiveUsers", [])
        for response_tuple in rows:
            my_active_users.append(response_tuple[0])
        return my_active_users

    def user_active_state_get(self, user_guid: str) -> bool:
        args: list[str, int] = [
            user_guid,
        ]
        rows: list[Tuple[int]] = self._call_proc("getUserActivityStateByGUID", args, True)
        if len(rows) != 1:
            raise IncorrectRowCountError(f"Result count was not exactly one. Received: {rows}")
        if rows[0][0] == 0:
            active: bool = False
        else:
            active: bool = True
        return active

    def user_active_state_set(self, user_guid: str, active: bool) -> None:
        args = [
            user_guid,
        ]
        if active:
            proc = "setUserActiveByGUID"
        else:
            proc = "setUserInactiveByGUID"
        self._call_proc(name=proc, args=args)

    def user_create(self) -> str:
        rows: list[Tuple[str]] = self._call_proc("createUser", [])
        if len(rows) != 1:
            raise IncorrectRowCountError(f"Result count was not exactly one. Received: {rows}")
        return rows[0][0]

    def user_delete(self, user_guid: str) -> None:
        args: list = [
            user_guid,
        ]
        self._call_proc("deleteUserByGUID", args)

    def user_features_get(self, user_guid: str) -> list[str]:
        features: list[str] = []

        args: list = [
            user_guid,
        ]
        rows: list[Tuple[str]] = self._call_proc("getUserFeaturesByGUID", args)
        for row in rows:
            features.append(row[0])
        return features

    def user_income_get(self, user_guid: str) -> list[UserIncome]:
        income_sources: list[UserIncome] = []

        args: list[str] = [
            user_guid,
        ]
        rows: list[Tuple[int, str, Optional[str]]] = self._call_proc("getUserIncomeByGUID", args)
        for row in rows:
            income: UserIncome = UserIncome(amount=row[0], name=row[1], description=row[2])
            income_sources.append(income)
        return income_sources

    def user_income_set(self, user_guid: str, transaction: UserIncomeSetRequest) -> None:
        args = [user_guid, transaction.income_source_name, transaction.amount]
        self._call_proc("deltaUserIncomeByNameAndGUID", args)

    def user_merchant_transforms_get(self, user_guid: str) -> list[str]:
        merchants: list[str] = []

        args: list = [
            user_guid,
        ]
        rows: list[Tuple[str]] = self._call_proc("getUserMerchantTransformsByGUID", args)
        for row in rows:
            merchants.append(row[0])
        return merchants

    def user_notifications_get(self, user_guid: str) -> list[UserNotification]:
        notifications: list[UserNotification] = []

        args: list = [
            user_guid,
        ]
        rows: list[Tuple[int, datetime, str]] = self._call_proc("getUserNotificationByGUID", args)
        for row in rows:
            notification: UserNotification = UserNotification(
                index=row[0], timestamp=row[1], event=UserEvent.parse_raw(row[2])
            )
            notifications.append(notification)
        return notifications

    def user_population_by_guid_get(self, user_guid: str) -> int:
        rows: list[Tuple[int]] = self._call_proc(
            "getUserPopulationByGUID",
            [
                user_guid,
            ],
        )
        return rows[0][0]

    def user_stores_get(self, user_guid: str) -> list[UserStore]:
        stores: list[UserStore] = []

        args: list[str, int] = [
            user_guid,
        ]
        rows: list[Tuple[str, Optional[str], int]] = self._call_proc("getUserStoresByGUID", args)
        for row in rows:
            store: UserStore = UserStore(name=row[0], description=row[1], amount=row[2])
            stores.append(store)
        return stores

    def user_transport(self, user_guid: str, location: str) -> UserFeature:
        args: list = [user_guid, location]
        rows: list[Tuple[str, Optional[str]]] = self._call_proc("transportUserByGUID", args)
        if len(rows) != 1:
            raise IncorrectRowCountError(f"Result count was not exactly one. Received: {rows}")
        location_tuple: Tuple[str, Optional[str]] = rows[0]
        location: UserFeature = UserFeature(name=location_tuple[0], description=location_tuple[1])
        return location

    # Utility functions

    def process_action_queue(self, action_queue: ActionQueue) -> None:
        for action in action_queue.queue:
            self._call_proc(action.name, action.arguments)

    # pylint: disable=duplicate-code
    def _call_proc(self, name: str, args: list, debug: bool = False) -> Optional[list[Tuple]]:
        if debug:
            logging.debug("[DAO] [Stored Proc Call Details] Name: {%s}, Arguments: {%s}", name, args)
        rows: Optional[list[Tuple]] = None
        try:
            cnx = self.cnxpool.get_connection()
            cursor = cnx.cursor()
            cursor.callproc(name, args)
            for result in cursor.stored_results():
                rows = result.fetchall()
            cursor.close()
        except socket.error as error:
            logging.debug(error)
            raise error
        except mysql.connector.Error as error:
            if error.errno == errorcode.ER_ACCESS_DENIED_ERROR:
                logging.debug("Something is wrong with your user name or password")
            elif error.errno == errorcode.ER_BAD_DB_ERROR:
                logging.debug("Database does not exist")
            else:
                logging.debug(error)
            raise error
        else:
            cnx.close()

        if debug:
            logging.debug("[DAO] [Stored Proc Call Details] Returning:")
            logging.debug(rows)
        return rows



from rowantree.contracts import UserFeature

from ..contracts.requests.user_transport_request import UserTransportRequest
from .abstract_controller import AbstractController


class UserTransportController(AbstractController):
    def execute(self, user_guid: str, request: UserTransportRequest) -> UserFeature:
        return self.dao.user_transport(user_guid=user_guid, location=request.location)



from ..contracts.responses.user_create_response import UserCreateResponse
from .abstract_controller import AbstractController


class UserCreateController(AbstractController):
    def execute(self) -> UserCreateResponse:
        return UserCreateResponse(guid=self.dao.user_create())



import logging

from ..contracts.responses.user_active_get_response import UserActiveGetResponse
from ..db.dao import DBDAO
from ..db.incorrect_row_count_error import IncorrectRowCountError
from .abstract_controller import AbstractController


class UserActiveGetController(AbstractController):
    def __init__(self, dao: DBDAO):
        super().__init__(dao=dao)

    def execute(self, user_guid: str) -> UserActiveGetResponse:
        # If the requested user does not exist we do not expose this in the response. (information leakage).
        # If the user is not found or is inactive we return an inactive response.
        try:
            user_active_state: bool = self.dao.user_active_state_get(user_guid=user_guid)
            logging.debug("user state requested for: {%s}, result: {%i}", user_guid, user_active_state)
        except IncorrectRowCountError as error:
            logging.debug("caught: {%s}", str(error))
            user_active_state: bool = False

        return UserActiveGetResponse(active=user_active_state)



from abc import ABC, abstractmethod
from typing import Any, Optional

from ..db.dao import DBDAO


class AbstractController(ABC):
    dao: DBDAO

    def __init__(self, dao: DBDAO):
        self.dao = dao

    @abstractmethod
    def execute(self, *args, **kwargs) -> Optional[Any]:
        """Should be implemented in the subclass"""



from ..contracts.requests.user_active_set_request import UserActiveSetRequest
from ..db.dao import DBDAO
from .abstract_controller import AbstractController


class UserActiveSetController(AbstractController):
    def __init__(self, dao: DBDAO):
        super().__init__(dao=dao)

    def execute(self, user_guid: str, request: UserActiveSetRequest) -> UserActiveSetRequest:
        self.dao.user_active_state_set(user_guid=user_guid, active=request.active)
        return request



import logging
import os
from pathlib import Path

from fastapi import FastAPI, Header, status
from mysql.connector.pooling import MySQLConnectionPool
from starlette.exceptions import HTTPException
from starlette.middleware.cors import CORSMiddleware

from rowantree.contracts import ActionQueue, UserFeature, WorldStatus
from rowantree.contracts.dto.user.state import UserState

from .config.server import ServerConfig
from .contracts.requests.merchant_transform_request import MerchantTransformRequest
from .contracts.requests.user_active_set_request import UserActiveSetRequest
from .contracts.requests.user_income_set_request import UserIncomeSetRequest
from .contracts.requests.user_transport_request import UserTransportRequest
from .contracts.responses.user_active_get_response import UserActiveGetResponse
from .contracts.responses.user_create_response import UserCreateResponse
from .contracts.responses.user_features_get_response import UserFeaturesGetResponse
from .contracts.responses.user_income_get_response import UserIncomeGetResponse
from .contracts.responses.user_merchant_transforms_get_response import UserMerchantTransformsGetResponse
from .contracts.responses.user_population_get_response import UserPopulationGetResponse
from .contracts.responses.user_stores_get_response import UserStoresGetResponse
from .controllers.action_queue_process import ActionQueueProcessController
from .controllers.merchant_transforms_perform import MerchantTransformPerformController
from .controllers.user_active_get import UserActiveGetController
from .controllers.user_active_set import UserActiveSetController
from .controllers.user_create import UserCreateController
from .controllers.user_delete import UserDeleteController
from .controllers.user_features_active_get import UserFeaturesActiveGetController
from .controllers.user_features_get import UserFeaturesGetController
from .controllers.user_income_get import UserIncomeGetController
from .controllers.user_income_set import UserIncomeSetController
from .controllers.user_merchant_transforms_get import UserMerchantTransformsGetController
from .controllers.user_population_get import UserPopulationGetController
from .controllers.user_state_get import UserStateGetController
from .controllers.user_stores_get import UserStoresGetController
from .controllers.user_transport import UserTransportController
from .controllers.world_get import WorldStatusGetController
from .db.dao import DBDAO
from .db.utils import get_connect_pool

# Generating server configuration
config: ServerConfig = ServerConfig()

# Setup logging
Path(config.log_dir).mkdir(parents=True, exist_ok=True)
logging.basicConfig(
    format="%(asctime)s - %(levelname)s - %(message)s",
    datefmt="%m/%d/%Y %I:%M:%S %p",
    level=logging.DEBUG,
    filemode="w",
    filename=f"{config.log_dir}/{os.uname()[1]}.therowantree.service.log",
)

logging.debug("Starting server")

logging.debug(config.json(by_alias=True, exclude_unset=True))

# Creating database connection pool, and DAO
cnxpool: MySQLConnectionPool = get_connect_pool(config=config)
dao: DBDAO = DBDAO(cnxpool=cnxpool)

# Create controllers
merchant_transforms_perform_controller = MerchantTransformPerformController(dao=dao)
user_active_get_controller = UserActiveGetController(dao=dao)
user_active_set_controller = UserActiveSetController(dao=dao)
user_create_controller = UserCreateController(dao=dao)
user_delete_controller = UserDeleteController(dao=dao)
user_features_get_controller = UserFeaturesGetController(dao=dao)
user_features_active_get_controller = UserFeaturesActiveGetController(dao=dao)
user_income_get_controller = UserIncomeGetController(dao=dao)
user_income_set_controller = UserIncomeSetController(dao=dao)
user_merchant_transforms_get_controller = UserMerchantTransformsGetController(dao=dao)
user_population_get_controller = UserPopulationGetController(dao=dao)
user_transport_controller = UserTransportController(dao=dao)
user_state_get_controller = UserStateGetController(dao=dao)
user_stores_get_controller = UserStoresGetController(dao=dao)
world_status_get_controller = WorldStatusGetController(dao=dao)
action_queue_process_controller = ActionQueueProcessController(dao=dao)

# Create the FastAPI application
app = FastAPI()

#  Apply COR Configuration | https://fastapi.tiangolo.com/tutorial/cors/
origins = ["http://localhost", "http://localhost:8080", "*"]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


# Naive auth system until idp can be introduced
def authorize(api_access_key: str):
    if api_access_key != config.access_key:
        logging.debug("bad access key")
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="Bad Access Key")


# Define our handlers

# Application Health Endpoint
@app.get("/health/plain", status_code=status.HTTP_200_OK)
async def health_plain() -> bool:
    return True


# Request Merchant Exchange (transform) For User
@app.post("/v1/user/{user_guid}/merchant", status_code=status.HTTP_201_CREATED)
async def merchant_transforms_perform_handler(
    user_guid: str, request: MerchantTransformRequest, api_access_key: str = Header(default=None)
) -> None:
    authorize(api_access_key=api_access_key)
    merchant_transforms_perform_controller.execute(user_guid=user_guid, request=request)


@app.get("/v1/user/{user_guid}/merchant", status_code=status.HTTP_200_OK)
async def user_merchant_transforms_get_handler(
    user_guid: str, api_access_key: str = Header(default=None)
) -> UserMerchantTransformsGetResponse:
    authorize(api_access_key=api_access_key)
    return user_merchant_transforms_get_controller.execute(user_guid=user_guid)


# Get User's Active State
@app.get("/v1/user/{user_guid}/active", status_code=status.HTTP_200_OK)
async def user_active_get_handler(user_guid: str, api_access_key: str = Header(default=None)) -> UserActiveGetResponse:
    authorize(api_access_key=api_access_key)
    return user_active_get_controller.execute(user_guid=user_guid)


# Set User's Active State
@app.post("/v1/user/{user_guid}/active", status_code=status.HTTP_200_OK)
async def user_active_set_handler(
    user_guid: str, request: UserActiveSetRequest, api_access_key: str = Header(default=None)
) -> UserActiveSetRequest:
    authorize(api_access_key=api_access_key)
    return user_active_set_controller.execute(user_guid=user_guid, request=request)


# Create User
@app.post("/v1/user", status_code=status.HTTP_201_CREATED)
async def user_create_handler(api_access_key: str = Header(default=None)) -> UserCreateResponse:
    authorize(api_access_key=api_access_key)
    return user_create_controller.execute()


# Delete User
@app.delete("/v1/user/{user_guid}", status_code=status.HTTP_200_OK)
async def user_delete_handler(user_guid: str, api_access_key: str = Header(default=None)) -> None:
    authorize(api_access_key=api_access_key)
    user_delete_controller.execute(user_guid=user_guid)


@app.get("/v1/user/{user_guid}/features", status_code=status.HTTP_200_OK)
async def user_features_get_handler(
    user_guid: str, api_access_key: str = Header(default=None)
) -> UserFeaturesGetResponse:
    authorize(api_access_key=api_access_key)
    return user_features_get_controller.execute(user_guid=user_guid)


@app.get("/v1/user/{user_guid}/features/active", status_code=status.HTTP_200_OK)
async def user_features_active_get_handler(
    user_guid: str, api_access_key: str = Header(default=None), details: bool = False
) -> UserFeature:
    authorize(api_access_key=api_access_key)
    return user_features_active_get_controller.execute(user_guid=user_guid, details=details)


@app.get("/v1/user/{user_guid}/income", status_code=status.HTTP_200_OK)
async def user_income_get_handler(user_guid: str, api_access_key: str = Header(default=None)) -> UserIncomeGetResponse:
    authorize(api_access_key=api_access_key)
    return user_income_get_controller.execute(user_guid=user_guid)


@app.post("/v1/user/{user_guid}/income", status_code=status.HTTP_200_OK)
async def user_income_set_handler(
    user_guid: str, request: UserIncomeSetRequest, api_access_key: str = Header(default=None)
) -> None:
    authorize(api_access_key=api_access_key)
    user_income_set_controller.execute(user_guid=user_guid, request=request)


@app.get("/v1/user/{user_guid}/population", status_code=status.HTTP_200_OK)
async def user_population_get_handler(
    user_guid: str, api_access_key: str = Header(default=None)
) -> UserPopulationGetResponse:
    authorize(api_access_key=api_access_key)
    return user_population_get_controller.execute(user_guid=user_guid)


@app.post("/v1/user/{user_guid}/transport", status_code=status.HTTP_200_OK)
async def user_transport_handler(
    user_guid: str, request: UserTransportRequest, api_access_key: str = Header(default=None)
) -> UserFeature:
    authorize(api_access_key=api_access_key)
    return user_transport_controller.execute(user_guid=user_guid, request=request)


@app.get("/v1/user/{user_guid}/state", status_code=status.HTTP_200_OK)
async def user_state_get_handler(user_guid: str, api_access_key: str = Header(default=None)) -> UserState:
    authorize(api_access_key=api_access_key)
    return user_state_get_controller.execute(user_guid=user_guid)


@app.get("/v1/user/{user_guid}/stores", status_code=status.HTTP_200_OK)
async def user_stores_get_handler(user_guid: str, api_access_key: str = Header(default=None)) -> UserStoresGetResponse:
    authorize(api_access_key=api_access_key)
    return user_stores_get_controller.execute(user_guid=user_guid)


@app.get("/v1/world", status_code=status.HTTP_200_OK)
async def world_get_handler(api_access_key: str = Header(default=None)) -> WorldStatus:
    authorize(api_access_key=api_access_key)
    return world_status_get_controller.execute()


@app.post("/v1/world/queue", status_code=status.HTTP_200_OK)
async def action_queue_process_handler(action_queue: ActionQueue, api_access_key: str = Header(default=None)) -> None:
    authorize(api_access_key=api_access_key)
    return action_queue_process_controller.execute(action_queue=action_queue)



from ..contracts.responses.user_stores_get_response import UserStoresGetResponse
from .abstract_controller import AbstractController


class UserStoresGetController(AbstractController):
    def execute(self, user_guid: str) -> UserStoresGetResponse:
        return UserStoresGetResponse(stores=self.dao.user_stores_get(user_guid=user_guid))



import configparser
import os
from typing import Optional

from pydantic import BaseModel


class ServerConfig(BaseModel):
    log_dir: Optional[str]

    access_key: Optional[str]

    database_server: Optional[str]
    database_name: Optional[str]
    database_username: Optional[str]
    database_password: Optional[str]

    def __init__(self, *args, config_file_path: str = "rowantree.config", **kwargs):
        super().__init__(*args, **kwargs)
        config = configparser.ConfigParser()
        config.read(config_file_path)

        # Directory Options
        self.log_dir = config.get("DIRECTORY", "logs_dir")

        # Server Options
        self.access_key = config.get("SERVER", "access_key")

        # Database Options
        self.database_server = config.get("DATABASE", "server")
        self.database_name = config.get("DATABASE", "database")
        self.database_username = config.get("DATABASE", "username")
        self.database_password = config.get("DATABASE", "password")

        if "LOGS_DIR" in os.environ:
            self.log_dir = os.environ["LOGS_DIR"]

        if "ACCESS_KEY" in os.environ:
            self.access_key = os.environ["ACCESS_KEY"]

        if "DATABASE_SERVER" in os.environ:
            self.database_server = os.environ["DATABASE_SERVER"]

        if "DATABASE_NAME" in os.environ:
            self.database_name = os.environ["DATABASE_NAME"]

        if "DATABASE_USERNAME" in os.environ:
            self.database_username = os.environ["DATABASE_USERNAME"]

        if "DATABASE_PASSWORD" in os.environ:
            self.database_password = os.environ["DATABASE_PASSWORD"]



from ..contracts.responses.user_population_get_response import UserPopulationGetResponse
from .abstract_controller import AbstractController


class UserPopulationGetController(AbstractController):
    def execute(self, user_guid: str) -> UserPopulationGetResponse:
        return UserPopulationGetResponse(population=self.dao.user_population_by_guid_get(user_guid=user_guid))
